package Servlets;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.charset.Charset;

public class AreaFormServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        resp.setContentType("text/html");
        resp.setCharacterEncoding("utf-8");

        try {
            resp.getOutputStream().write(generateHTML(req));
            resp.getOutputStream().flush();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    byte[] generateHTML(HttpServletRequest request) {
        byte[] res = null;
        float x=-6,y=-6,r=-6;
        boolean isValid = true;

        try {
            x = Float.parseFloat(request.getParameter("x_pos"));
            y = Float.parseFloat(request.getParameter("y_pos"));
            r = Float.parseFloat(request.getParameter("radius"));
        } catch (NumberFormatException ex) {
            isValid = false;
        }

        if(isValid && validateData(x,y,r))
        {
            res = genSuccPage(x,y,r, inRegion(x,y,r) );
        }
        else
        {
            res = genErrorPage();
        }


        return res;
    }

    boolean validateData(float x, float y, float r) {
        boolean res = true;

        if(x > 3 || x < -5)
            res = false;
        if(y < -5 || y > 3)
            res = false;
        if(r < 1 || r > 3)
            res = false;

        return res;
    }

    byte[] genErrorPage()
    {
        byte[] ret = null;

        try
        {
/*
            ret = Files.readAllBytes(Paths.get(this.getClass().getResource("/templates/errTemplate.html").toURI()));
*/
            FileInputStream f_in = new FileInputStream(this.getClass().getResource("/templates/errTemplate.html").getFile());
            ByteArrayOutputStream buff = new ByteArrayOutputStream();

            int c;

            while((c = f_in.read()) != -1)
                buff.write(c);

            ret = buff.toByteArray();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }

        return ret;
    }

    byte[] genSuccPage(float x, float y, float r, boolean isInRegion)
    {
        byte[] res = null;
        try
        {
/*
            String myHTML = new String(Files.readAllBytes(Paths.get(this.getClass().getResource("/templates/succTemplate.html").toURI())), StandardCharsets.UTF_8);
*/
            FileInputStream f_in = new FileInputStream(this.getClass().getResource("/templates/succTemplate.html").getFile());
            ByteArrayOutputStream buff = new ByteArrayOutputStream();

            int c;

            while((c = f_in.read()) != -1)
                buff.write(c);

            String myHTML = new String(buff.toByteArray(), Charset.forName("utf-8"));
            String replaceString;
            if(isInRegion)
                replaceString = "Точка ("+ x +", "+ y + ")" +" в области <br/>" + "Радиус " + r;
            else
                replaceString = "Точка ("+ x +", "+ y + ")" +" вне области <br />" + "Радиус " + r;

            myHTML = myHTML.replaceAll("(.*)__RESULT_PLACE(.*)", replaceString);

            res = myHTML.getBytes();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        return  res;
    }

    boolean inRegion(float x, float y,float r)
    {
        boolean res = false;

        if(x >= 0)
        {
            if(y >= 0 )
                res = FirstQuarter(x,y,r);
        }
        else
        {
            if(y >= 0)
                res = SecondQuarter(x,y,r);
            else
                res = ThirdQuarter(x,y,r);
        }

        return res;
    }

    boolean FirstQuarter(float x, float y, float r)
    {
        boolean ret = true;

        if(y > (x*(-2) + 1))
            ret = false;

        return ret;
    }

    boolean SecondQuarter(float x, float y, float r)
    {
        boolean ret = true;

        if(Math.pow(x,2) + Math.pow(y,2) > Math.pow(r,2))
            ret = false;

        return ret;
    }

    boolean ThirdQuarter(float x, float y, float r)
    {
        boolean ret = true;
        if(x < -r || y < -r)
            ret = false;

        return ret;
    }
}