package RegLocation.Client;


import RegLocation.Lib.Mark;
import RegLocation.Lib.MarkStates;

import java.awt.*;
import java.util.HashMap;
import java.util.Vector;

public class SilhouetteView
{
    Vector<Mark> SAL;
    HashMap<Integer,MarkStates> Colors;
    SizesConverter SizesConverter;
    float MarkMaxR = 10;

    public SilhouetteView(Vector<Mark> SAL,SizesConverter convert, HashMap<Integer,MarkStates> colors)
    {
        this.SAL = SAL;
        SizesConverter = convert;
        this.Colors = colors;
    }

    public void paint(Graphics g)
    {
        g.setColor(Color.WHITE);
        g.fillRect(0,0,(int)SizesConverter.getWidth(),(int)SizesConverter.getHeight());
        paintSilhouette(g);
        paintAxes(g);
        paintPoints(g);
    }

    void paintSilhouette(Graphics g)
    {
        int R       = SizesConverter.toPx(SilhouetteComponent.R);
        int HalfR   = SizesConverter.toPx(SilhouetteComponent.R/2);
        int XCenter = SizesConverter.getCenterX();
        int YCenter = SizesConverter.getCenterY();

        g.setColor(Color.BLUE);
        g.fillRect(XCenter-R,YCenter-HalfR,R, HalfR);
        g.fillArc(XCenter-HalfR,YCenter-HalfR,R,R,0,-90);
        g.fillPolygon(getTrianglePolygon(R));
    }

    void paintAxes(Graphics g)
    {
        int R       = SizesConverter.toPx(SilhouetteComponent.R);
        int HalfR   = SizesConverter.toPx(SilhouetteComponent.R/2);
        int XCenter = SizesConverter.getCenterX();
        int YCenter = SizesConverter.getCenterY();

        g.setColor(Color.BLACK);
        g.drawLine(0, YCenter, 2*XCenter,YCenter);
        g.drawLine(XCenter, 0, XCenter, 2*YCenter);

        g.drawLine(XCenter - R, YCenter + 5, XCenter - R, YCenter - 5);
        g.drawLine(XCenter + R, YCenter - 5, XCenter + R, YCenter + 5);
        g.drawLine(XCenter + 5, YCenter + R, XCenter - 5, YCenter + R);
        g.drawLine(XCenter + 5, YCenter - R, XCenter - 5, YCenter - R);

        g.drawLine(XCenter - HalfR, YCenter + 5, XCenter - HalfR, YCenter - 5);
        g.drawLine(XCenter + HalfR, YCenter - 5, XCenter + HalfR, YCenter + 5);
        g.drawLine(XCenter + 5, YCenter + HalfR, XCenter - 5, YCenter + HalfR);
        g.drawLine(XCenter + 5, YCenter - HalfR, XCenter - 5, YCenter - HalfR);
    }

    public void paintPoints(Graphics g)
    {
        Mark[] SAL_array = SAL.toArray(new Mark[SAL.size()]);
        for(Mark item : SAL_array)
        {
            int item_x,item_y;
            float factor = item.incFactor();
            item_x =  SizesConverter.getCenterX() + SizesConverter.toPx(item.getCoord()[0]);
            item_y =  SizesConverter.getCenterY() - SizesConverter.toPx(item.getCoord()[1]);
            int radius = (int)(MarkMaxR*factor);

            MarkStates MarkState = Colors.get(item.hashCode());
            Color MarkColor = null;
            if( MarkState == MarkStates.InRegion)
                MarkColor = Color.GREEN;
            else if(MarkState == MarkStates.OutOfRegion)
                MarkColor = Color.RED;
            else
                MarkColor = Color.GRAY;
            g.setColor(MarkColor);
            g.fillOval(item_x - radius/2,item_y - radius/2, radius,radius);
        }
    }

    Polygon getTrianglePolygon(int R)
    {
        int XCenter = SizesConverter.getCenterX();
        int YCenter = SizesConverter.getCenterY();

        int num_points = 3;
        int[] x_points = new int[num_points];
        int[] y_points = new int[num_points];

        x_points[0] = XCenter - R;
        y_points[0] = YCenter;

        x_points[1] = XCenter;
        y_points[1] = YCenter;

        x_points[2] = XCenter;
        y_points[2] = YCenter + R;

        return new Polygon(x_points,y_points,num_points);
    }
}
