package RegLocation.Lib;

import java.net.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;


public class SilhouetteChannel
{
    Socket _sck;
    SocketAddress _endpoint;

    public SilhouetteChannel(SocketAddress endpoint)
    {
        try
        {
            _endpoint = endpoint;
            _sck = new Socket();
            _sck.connect(endpoint);
        }
        catch(SocketException se)
        {
            tryReconect();
        }
        catch(IOException ex)
        {
            ex.printStackTrace();
        }
    }

    public SilhouetteChannel(Socket lsocket)
    {
        _endpoint = null;
        _sck      = lsocket;
    }


    public void sendRequest(Request request) throws  IOException
    {
        boolean error_flag;
        do
        {
            error_flag = false;
            try
            {
            OutputStream sckStream = _sck.getOutputStream();
            request.getBytesStream(sckStream);
            }
            catch (SocketException se)
            {
                tryReconect();
                error_flag = true;
            }
        }while (error_flag);
    }

    public boolean sendResponse(Response response) throws IOException
    {
        boolean return_value  = true;
        try
        {
            OutputStream sckStream = _sck.getOutputStream();
            response.getBytesStream(sckStream);
        }
        catch (SocketException se)
        {
            return_value = false;
            return return_value;
        }

        return return_value;
    }

    public Response receiveResponse()
    {
        Response ret = null;
        try
        {
            InputStream sckStream = _sck.getInputStream();
            ret = Response.toResponse(sckStream);
        }
        catch (SocketException ex)
        {
            return null;
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        return ret;
    }

    public Request receiveRequest() throws IOException
    {
        Request ret = null;
        try
        {
            InputStream stream = _sck.getInputStream();
            ret = Request.toRequest(stream);
        }
        catch (SocketException se)
        {
            return null;
        }
        return ret;
    }

    boolean tryReconect()
    {
        boolean flag = false;
        boolean itFirst = true;
        do
        {
            if(!itFirst)
                try
                {
                    Thread.sleep(100);
                }
                catch (InterruptedException ex)
                {
                    Thread.currentThread().interrupt();
                }
            itFirst = false;
            flag    = false;
            try
            {
                _sck = new Socket();
                _sck.connect(_endpoint,100);
            }
            catch (SocketException ex)
            {
                flag = true;
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }
        } while (flag);
        return true;
    }

}

