package RegLocation.Server;

import RegLocation.Lib.Request;
import RegLocation.Lib.Response;
import RegLocation.Lib.SilhouetteChannel;

import java.io.IOException;
import java.net.Socket;
import java.util.Vector;


public class ClientSession extends Thread
{
    SilhouetteChannel channel;
    boolean stopping;
    Vector<Response> responseVector;

    public ClientSession(Socket socketr)
    {
        stopping = false;
        channel = new SilhouetteChannel(socketr);
        responseVector = new Vector<Response>();

        new Thread(new Runnable() { //Responses sender
            @Override
            public void run() {
                while(!stopping)
                {
                    Response[] responses = responseVector.toArray(new Response[responseVector.size()]);
                    for(Response item : responses)
                    {
                        try
                        {
                            if(!channel.sendResponse(item)) {
                                toStop();
                                return;
                            }
                            responseVector.remove(item);
                        }
                        catch(IOException ex)
                        {
                            ex.printStackTrace();
                        }
                    }

                    try
                    {
                        Thread.sleep(100);
                    }
                    catch (InterruptedException ex)
                    {
                        Thread.currentThread().interrupt();
                    }

                    Thread.yield();
                }
            }
        }).start();
    }

    @Override
    public void run()
    {
        while(!stopping)
        {
            Request request = null;
            try
            {
               request = channel.receiveRequest();
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }

            if(request == null)
            {
                toStop();
                continue;
            }

            new SilhouetteProcThread(this,request).start();

            try
            {
                Thread.sleep(100);
            }
            catch (InterruptedException ex)
            {
                Thread.currentThread().interrupt();
            }
        }
    }

    public void toStop()
    {
        stopping = true;
        System.out.println("Going down");
    }

    public void saveResponse(Response rresponse)
    {
        responseVector.add(rresponse);
    }
}
